@@ -13,9 +13,12 @@ sub set_sent { ++$_[0]->{set_sent} }
 sub set_seen { ++$_[0]->{set_seen} }
 
 sub check_done {
+    no warnings;
+#    $_[0]->{set_seen} and 
     $_[0]->{set_sent} == $_[0]->{set_seen}
-	and
-	$_[0]->{get_sent} == $_[0]->{get_seen}
+    and
+#	$_[0]->{get_seen} and
+        $_[0]->{get_sent} == $_[0]->{get_seen}
 }
 
 1;
